var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var http_1 = require('angular2/http');
var core_1 = require('angular2/core');
var logger_1 = require('../logger/logger');
var credentials_1 = require('./credentials');
var ng2_cookies_1 = require('ng2-cookies/ng2-cookies');
var API_URL = '/unityapi/api/Authentication';
var AuthenticationService = (function () {
    function AuthenticationService(http, logger) {
        this.http = http;
        this.logger = logger;
        this.AUTH_COOKIE = '.unity.com';
        this.locationWatcher = new core_1.EventEmitter();
        this.expires = 0;
        this.intervalId = null;
        this.expiresTimerId = null;
        this.COOKIE_EXPIRATION_DAYS = 1;
        this.credentials = this.credentials || new credentials_1.Credentials();
    }
    AuthenticationService.prototype.authenticate = function (username, password) {
        var _this = this;
        var headers = new http_1.Headers();
        var params = JSON.stringify({ 'UserName': username, 'Password': password });
        headers.append('content-type', 'application/json; charset=utf-8');
        this.logger.info('Calling authenticate() with ' + username + ', ' + password);
        return this.http.post(API_URL, params, {
            headers: headers
        }).map(function (response) {
            // get the bearer token from the response;
            _this.logger.info('RESPONSE TEXT:' + JSON.stringify(response));
            _this.credentials = JSON.parse(response.text());
            _this.logger.info('AUTHENTICATED: ' + JSON.stringify(_this.credentials));
            _this.authenticated = true;
            _this.logger.info('setting authenticated to ' + _this.authenticated);
            _this._storeCredentials(_this.credentials);
            return _this.credentials;
        }, function (error) {
            _this.credentials = null;
            _this.logger.error('ERROR: ' + JSON.stringify(error));
        });
    };
    AuthenticationService.prototype.recover = function (email) {
        var _this = this;
        var headers = new http_1.Headers();
        headers.append('content-type', 'application/json; charset=utf-8');
        return this.http.put(API_URL + '?email=' + email, null, {
            headers: headers
        }).map(function (response) {
            return response;
        }, function (error) {
            _this.logger.error('ERROR: ' + JSON.stringify(error));
            return error;
        });
    };
    AuthenticationService.prototype.getCredentials = function () {
        return this._getCredentials();
    };
    AuthenticationService.prototype.logout = function () {
        this.credentials = new credentials_1.Credentials();
        this._clearCredentials();
        this.authenticated = false;
        this._emitAuthStatus(true);
    };
    AuthenticationService.prototype.subscribe = function (onNext, onThrow, onReturn) {
        return this.locationWatcher.subscribe(onNext, onThrow, onReturn);
    };
    AuthenticationService.prototype.isAuthenticated = function () {
        this._getCredentials();
        this.logger.info('this user is ' + (this.authenticated ? '' : 'not') + ' authenticated');
        return this.authenticated;
    };
    AuthenticationService.prototype._getCredentials = function () {
        // Try to get credentials from memory. If it's not there,
        // try to load it from cookies. 
        if (this.credentials === undefined || this.credentials.access_token.length === 0) {
            var authCookie = ng2_cookies_1.Cookie.getCookie(this.AUTH_COOKIE);
            this.credentials = JSON.parse(authCookie);
        }
        this.credentials = this.credentials || new credentials_1.Credentials();
        this.authenticated = this.credentials.access_token.length > 0;
        return this.credentials;
    };
    AuthenticationService.prototype._emitAuthStatus = function (success) {
        this.locationWatcher.emit({ success: success, authenticated: this.authenticated, token: this.credentials.access_token, expires: this.expires });
    };
    AuthenticationService.prototype._startExpiresTimer = function (seconds) {
        if (this.expiresTimerId != null) {
            clearTimeout(this.expiresTimerId);
        }
        this.expiresTimerId = setTimeout(function () {
            console.log('Session has expired');
            //this.doLogout();
        }, seconds * 1000); // seconds * 1000
        console.log('Token expiration timer set for', seconds, "seconds");
    };
    AuthenticationService.prototype._clearCredentials = function () {
        ng2_cookies_1.Cookie.deleteCookie(this.AUTH_COOKIE);
    };
    AuthenticationService.prototype._storeCredentials = function (c) {
        ng2_cookies_1.Cookie.setCookie(this.AUTH_COOKIE, JSON.stringify(c), this.COOKIE_EXPIRATION_DAYS, '/', this.AUTH_COOKIE);
    };
    AuthenticationService.prototype.ngOnInit = function () {
        this._getCredentials();
    };
    AuthenticationService = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [http_1.Http, logger_1.SDLogger])
    ], AuthenticationService);
    return AuthenticationService;
})();
exports.AuthenticationService = AuthenticationService;
//# sourceMappingURL=authentication.service.js.map