import {
  it,
  inject,
  injectAsync,
  describe,
  beforeEachProviders,
  TestComponentBuilder
} from 'angular2/testing';

import {Injector, Component, provide} from 'angular2/core';
import {Jsonp, ConnectionBackend, BaseRequestOptions, Response, ResponseOptions, Http} from 'angular2/http';
import {MockBackend, MockConnection} from 'angular2/http/testing';
import {Observable} from 'rxjs/Rx';
import {Credentials} from './credentials';
import { Cookie } from 'ng2-cookies/ng2-cookies';

// Load the implementations that should be tested
import {AuthenticationService} from './authentication.service';
import {SDLogger} from '../shared/services/logger';

describe('AuthenticationService', () => {
    
    // This will be the mock response used for this test. It is returned by the mock instance of http.
    let mockResponse = new Observable<Credentials>(() => {JSON.stringify({
            access_token: '0d0d9085-0c19-455b-9211-65de726bc10b',
            token_type: 'Bearer',
            expires_in: 1,
            refresh_token: '',
            scope: 'Api'
       })
    });
    
    let mockCode = new Observable<string>(() => {'12345'});
    
    var url = 'http://foo.bar';
    var http: Http;
    var injector: Injector;
    var backend: MockBackend;
    var baseResponse: Response;
    var jsonp: Jsonp;
    var authenticationService: AuthenticationService;
    
    // set up the mock services that will be used in these tests
    beforeEach(() => {
      injector = Injector.resolveAndCreate([
        BaseRequestOptions,
        MockBackend,
        Cookie,
        AuthenticationService,
        SDLogger,
        provide(
            Http,
            {
              useFactory: function(backend: ConnectionBackend, defaultOptions: BaseRequestOptions) {
                return new Http(backend, defaultOptions);
              },
              deps: [MockBackend, BaseRequestOptions]
            }),
        provide(
            Jsonp,
            {
              useFactory: function(backend: ConnectionBackend, defaultOptions: BaseRequestOptions) {
                return new Jsonp(backend, defaultOptions);
              },
              deps: [MockBackend, BaseRequestOptions]
            })
      ]);
      
      http = injector.get(Http);
      jsonp = injector.get(Jsonp);
      
      // override the response of all http requests so they return our mock response
       backend = injector.get(MockBackend);
        const baseResponse = new Response(new ResponseOptions({body: mockResponse}));
        backend.connections.subscribe((c: MockConnection) => c.mockRespond(baseResponse));
    });
    
    // verify that we have no open connections after each test
    afterEach(() => backend.verifyNoPendingRequests());

    // here begin the tests...
    it('.authenticate() should accept two string parameters and return a json response with a bearer token upon successful authenication', 
        inject([ ], (logger, authentication:AuthenticationService, cookies: Cookie) => {
        
            authenticationService = injector.get(AuthenticationService);
            logger = injector.get(SDLogger);
            cookies = injector.get(Cookie);
            
            authenticationService.authenticate('testUser', 'a1111111')
            .subscribe(
            (resp) => { 
               expect(resp.access_token.length).toBe(35);
               expect(authenticationService.isAuthenticated()).toBeTruthy();
               
               var cookie = Cookie.getCookie(authenticationService.AUTH_COOKIE);
                logger.info('USING COOKIE:' + cookie);
               var credentials = JSON.parse(cookie);
               expect(credentials.access_token.length).toBe(36);
            },
            (error) => {
              logger.info('LOGIN ERROR:' + error);
             // expect(1).toBe(2);
            });
        })
    );
    
    it('.recover() should accept an email parameters and return a json response with a code on success', 
        inject([ ], (authentication:AuthenticationService) => {
       
         
        
            authenticationService = injector.get(AuthenticationService);
            
            authenticationService.recover('testUser@ttt.com')
            .subscribe(
            (resp) => { 
               //expect(resp.text.length).toBe(5);
                //expect(resp.access_token.length).toBe(36);
            },
            (error) => {
            
            });
        })
    );
    
});
